/* bc635.h - Datum PCI Time/Frequency Processor Card header definitions */

/*
  *******************************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/


/*
modification history
--------------------
*/

#ifndef __INCbc635h
#define __INCbc635h

#ifdef __cplusplus
extern  "C"
{
#endif

#include "time.h"
#include "sys/times.h"


/* typedefs */

   typedef struct
   {
     UINT8 packet_length;
     UINT8 packet_id;
     UINT8 packet_data[256]; /* one bigger than necessary */
   } TFP_GPS_PACKET; 

   typedef struct
   {
     TFP_GPS_PACKET send;
     TFP_GPS_PACKET receive;
   } TFP_GPS_SENDRECEIVE_PACKET;

/*
 * ioctl struct for ioctl: RDTIMETV, RDTIMETVREQ, RDEVENTTV, RDEVENTTVREQ
*
 * When time format is 'unix', holds current time as
 * contained in major(TIME1) & minor(TIME0) time regs.
 *
 * tv.tv_sec = seconds from midnight Jan 1, 1970
 *
 * The status bits have meaning as follows:
 *
 *     b0: flywheeling state
 *            0: locked to reference
 *            1: flywheeling(not locked)
 *
 *     b1: time offset state
 *            0: < 5 usec for mode 0
 *               < 2 usec for other modes
 *            1: > 5 usec for mode 0
 *               > 2 usec for other modes
 *
 *     b2: frequency offset state
 *            0: < 2 usec for other modes
 *            1: > 5 x 10-8
 *
 */
    typedef struct
    {
        struct timeval tv;
        UINT8     hnsec;        /* hundreds of nsecs */
        UINT8     status;       /* b0: flywheeling state 0=locked to
                                 * reference 1=flywheeling(not locked)
                                 * 
                                 * b1: time offset state 0: < 5 usec for mode 0
                                 * < 2 usec for other modes 1: > 5 usec for
                                 * mode 0 > 2 usec for other modes
                                 * 
                                 * b2: frequency offset state < 2 usec for other
                                 * modes 1: > 5 x 10-8 */
    }       TFP_TV;

/*
 * ioctl struct for ioctl: RDTIME, RDTIMEREQ, RDEVENT, RDEVENTREQ
 *
 * When time format is 'decimal', holds current time as
 * contained in major(TIME1) & minor(TIME0) time regs.
 *
 * Time is retrieved in decimal format except
 * for usec,hnsec.
 */
    typedef struct
    {
        struct tm tm;
        UINT32    usec;
        UINT32    hnsec;          /* hundreds of nsecs */
        UINT8    status;
    }       TFP_TM;


    struct periodic
    {
        UINT8 sync1pps;       /* TRUE = sync periodic to 1pps */
        UINT16 n1;            /* divider n1 */
        UINT16 n2;            /* divider n2 */
    };

/* Additional Structures to interpret GETDATA command results */
/* GETDATA_UTCINFO 0x18 */
    struct gd_utcinfo
    {
        UINT8   enable;
        INT8    leapsec;
        INT8    leapevt;
        UINT32  evttime;
    };


/* GETDATA_TCGENOFF  0x1C */
    struct tcgenoffset
    {
        INT16 offset;         /* local offset -16 to +16 Hours  */
        UINT8 half_hour;      /*  1: half_hour, 0: no_half_hour */
    };

/* GETDATA_LOCTMOFF  0x1D */
    struct loctmoffset
    {
        INT16 offset;         /* Generator offset -16 to +16 Hours  */
        UINT8 half_hour;      /* flag: 1: half_hour, 0: no_half_hour */
    };

/* GETDATA_LEAPSEC   0x1E */
    struct leapseconds
    {
        INT8   ls_flag;        /* 1: Insertion, -1: Deletion, 0: Disable */
        UINT32 leap_tm;   /* Leap Event Time - unix format */
    };





/* GETDATA_FWVER 0x1F */
    struct gd_fwversion
    {
        UINT8  vmajor;         /* major version number */
        UINT8  vminor;         /* minor version number */
        UINT8  month;          /* month (1 - 12) */
        UINT8  day;            /* day (1 - 31 ) */
        UINT16 year;           /* year (e.g. 1994) */
    };

/* GETDATA_OSCCTL 0x23 */
    struct gd_oscdisctl
    {
        INT8    discctl;
        INT8    phasectl;
        UINT16  phasestep;
        INT16   gain;
        UINT32  con;
    };


/* GETDATA_DTFW   0x4F */
    struct dtfwrev
    {
        UINT8  dt01;           /* 'D' */
        UINT8  dt02;           /* 'T' */
        UINT8  dt03;           /* '6' */
        UINT8  dt04;           /* '0' */
        UINT8  dt05;           /* '0' */
        UINT8  dt06;           /* '0' */
        UINT8  dt07;           /* Rev */
        UINT8  dt08;           /* Rev */
        UINT8  dt09;           /* Rev */
        UINT8  dt10;           /* Rev */
        UINT8  dt11;           /* Rev */
    };

/* GETDATA_TFPMODEL  0xF6 */
    struct tfpmodel
    {
        UINT8  tfp1;           /* 'B' */
        UINT8  tfp2;           /* 'C' */
        UINT8  tfp3;           /* '6' */
        UINT8  tfp4;           /* '3' */
        UINT8  tfp5;           /* '5' or '7' */
        UINT8  tfp6;           /* 'P' */
        UINT8  tfp7;           /* 'C' */
        UINT8  tfp8;           /* 'I' */
    };




/* Passed as arg in the GETDATA (38) command */
    struct getdata_t
    {
        UCHAR  arg;            /* Get Data Command Argument */
        UCHAR  padding[3];
        union
        {
            UINT8  tmode;      /* GETDATA_MODE      0x10 */
            UINT8  tformat;    /* GETDATA_TFORMAT   0x11 */
            UINT16 year;       /* GETDATA_YEAR      0x13 */
            struct periodic period;     /* GETDATA_PER       0x14 */
            UINT8    tcformat;   /* GETDATA_TCFMT     0x15 */
            UINT8    tcmod;      /* GETDATA_TCMOD     0x16 */
            INT32     offset;     /* GETDATA_TOFF      0x17 */
            struct gd_utcinfo utc;      /* GETDATA_UTCINFO   0x18 */

            INT16  tcoutfmt;   /* GETDATA_TCOUTFMT  0x1B */
            struct tcgenoffset genoff;  /* GETDATA_TCGENOFF  0x1C */
            struct loctmoffset locoff;  /* GETDATA_LOCTMOFF  0x1D */
            struct leapseconds leapsec; /* GETDATA_LEAPSEC   0x1E */

            struct gd_fwversion version;        /* GETDATA_FWVER     0x1F */
            INT8    clksrc;     /* GETDATA_CLKSRC    0x20 */
            INT8    jsctl;      /* GETDATA_JAMSC     0x21 */
            struct gd_oscdisctl ocsctl; /* GETDATA_OSCCTL    0x23 */
            UINT16 davalue;    /* GETDATA_DAVAL     0x24 */

            UINT8  battstat;   /* GETDATA_BATTSTAT  0x26 */
            INT32     clkval;     /* GETDATA_CLKVAL    0x29 */
            struct dtfwrev dtfw;/* GETDATA_DTFW   0x4F */
            UINT16 assemb;     /* GETDATA_ASSEMB    0xF4 */
            UINT16 hwfab;      /* GETDATA_HWFAB     0xF5 */
            struct tfpmodel tfpm;       /* GETDATA_TFPMODEL  0xF6 */
            UINT32 serialnum;     /* GETDATA_SERIAL    0xFE */
        }       data;
    };


/* imports */

    IMPORT int tfpDrv (void);
    IMPORT STATUS tfpDevInstall (int index, int intvec);

/* defines */


#define TM_YEAR_BASE 1900
#define STFP_MAX_READ 80
#define STFP_MAX_WRITE 80


/**************************************************************/
/* IOCTL COMMANDS DUAL PORT RAM INTERFACE COMMANDS            */
/**************************************************************/

#define SELTIMINGMODE 1        /* 0x10 Set TFP Timing Mode */
/* arg values */
#define      MODE_TIMECODE 0
#define      MODE_FREERUN  1
#define      MODE_EXT1PPS  2
#define      MODE_RTC      3
#define      MODE_GPS      6

#define SELTIMEFORMAT 2        /* 0x11 Set Time Register Format */
/* arg values */
#define      TIME_DECIMAL 0
#define      TIME_BINARY  1


#define SETTIMETM  3           /* 0x12 Set Time, TM */
/* arg value - (TFP_TM) */

#define SETTIMETV  4           /* 0x12 Set Time, TV */
/* arg value - (TFP_TV) */

#define SETYEAR   5            /* 0x13 Set Year */
/* arg value - (UINT32) */

#define SETPERIODIC  6          /* 0x14 Set Periodic Output */
/* arg value - (struct periodic *) */
#define      PERIODIC_NOSYNC 0
#define      PERIODIC_SYNC 1
 
#define SELTCFORMAT  7         /* 0x15 Set Input Time Code Format */
/* arg values */
#define      TC_IRIGA  'A'
#define      TC_IRIGB  'B'
#define      TC_IEEE   'I'
#define      TC_NASA36 'N'

#define SELTCMOD  8            /* 0x16 Set Input Time Code Modulation */
/* arg values */
#define      MOD_AM    'M'
#define      MOD_DCLS  'D'

#define SETPROPDELAY    9       /* 0x17 Set Proprgation Delay */
/* arg value - (UINT32) */
 
#define GETDATA 10              /* 0x19 Request TFP Data */
/* arg value - (struct getdata_t *) */
#define     GETDATA_MODE  0x10
#define     GETDATA_TFORMAT  0x11
#define     GETDATA_YEAR  0x13
#define     GETDATA_PER   0x14
#define     GETDATA_TCFMT  0x15
#define     GETDATA_TCMOD  0x16
#define     GETDATA_TOFF  0x17
#define     GETDATA_UTCINFO  0x18        /* 0x18 Request UTC Time Data */
#define     GETDATA_TCOUTFMT 0x1B
#define     GETDATA_TCGENOFF 0x1C
#define     GETDATA_LOCTMOFF 0x1D
#define     GETDATA_LEAPSEC  0x1E
#define     GETDATA_FWVER  0x1F
#define     GETDATA_CLKSRC  0x20
#define     GETDATA_JAMSC  0x21
#define     GETDATA_OSCCTL  0x23
#define     GETDATA_DAVAL  0x24
#define     GETDATA_BATTSTAT 0x26        /* 0x26 Request Battery Status */
#define     GETDATA_CLKVAL  0x29         /* 0x29 Request Advance/retard clk */
#define     GETDATA_REQLOCTMFG 0x41      /* 0x41 Request Local Time Flag */    
#define     GETDATA_DTFW  0x4F           /* 0x4f Request FW Part Number  */
#define     GETDATA_ASSEMB 0xF4          /* 0xF4 Request Assembly Part   */
#define     GETDATA_HWFAB  0xF5          /* 0xF5 Request HW FAB Number   */
#define     GETDATA_TFPMODEL 0xF6        /* 0xF6 Request TFP Model ID    */
#define     GETDATA_SERIAL  0xFE         /* 0xFe Request Serial Number   */

#define SOFTWARERESET 11        /* 0x1A Software Reset */
/* arg value - na */
 
#define SETTCOUTFMT  12         /* 0x1B Set Time Code Output Format */
/* arg values */
#define      TC_OUT_IRIGB 0x42
#define      TC_OUT_IEEE  0x49
 
#define SETGENTMOFFSET 13       /* 0x1C Set Generator Time Offset */
/* arg - (struct tcgenoffset *) */

#define SETLOCTMOFFSET 14       /* 0x1D Set Local Time Offset */
/* arg - (struct loctmoffset *) */

#define SETLEAPSECEVENT 15      /* 0x1E Program Leap Second Event */
/* arg - (struct leapseconds *) */

#define CONTROLTIMEBASE 16     /* Various Time Control Cmds */
/* arg values */               /* 0x20, 0x21, 0x23          */
#define      CLOCK_INTERNAL 1
#define      CLOCK_EXTERNAL 2
#define      JAMSYNC_ENABLE 3
#define      JAMSYNC_DISABLE 4
#define      DISCIPLINE_ENABLE 5
#define      DISCIPLINE_DISABLE 6
#define      FORCE_JAMSYNC 7
#define      CLEAR_USEC  8

#define SETDAC   17           /* 0x24 Load D/A Converter */
/* arg value - (UINT32) */

#define SETDISCGAIN  18       /* 0x25 Set Disciplining Gain */
/* arg - (UINT32) */
 
#define SYNCRTC   19          /* 0x27 Sync RTC To External Time Data */
/* arg - na */
 
#define DISCBATT  20          /* 0x28 Disconnect Battery From RTC Chip */
/* arg - na */

#define SETCLKVAL  21         /* 0x29 Set Advance/Retard Clock Value */
/* arg - (UINT32) */
 
#define REQGPSPACKET 22       /* 0x30 Request Packet From GPS */

#define SENDGPSPACKET 23      /* 0x31 Send Packet to GPS */

#define MANREQGPSPACKET 24    /* 0x32 Manually Request Packet From GPS */
 
#define SETGPSTMFMT  25       /* 0x33 Set Gps Time Format */
/* arg values */
#define      UTC_FMT  0
#define      GPS_FMT  1
 
#define SETGPSMDFLG  26       /* 0x34 Set GPS Mode Flag */
/* arg values */
#define      GPS_FLG_DIS 0
#define      GPS_FLG_ENA 1
 
#define SETLOCTMFLG  27       /* 0x40 Set Local Time Flag */
/* arg values */
#define      LOC_OFF_DIS 0
#define      LOC_OFF_ENA 1
 
#define SETYRINCFLG  28       /* 0x42 Set Auto Year Increment */
/* arg values */
#define      YR_INC_DIS 0
#define      YR_INC_ENA 1
 

/**************************************************************/
/* IOCTL COMMANDS REGISTER INTERFACE           */ 
/**************************************************************/


#define TIMEREQUEST  40        /* Request Time (Latch Time) */
/* arg value - na */

#define EVENTREQUEST 41        /* Request Event (Latch Event) */
/* arg value - na */

#define RDTIMETM   42            /* Read Time, Unlatched, Decimal */
/* arg value (TFP_TM *) */

#define RDEVENTTM   43           /* Read Event, Unlatched, Decimal */
/* arg value (TFP_TM *) */

#define RDTIMETV  44           /* Read Time, Unlatched, Binary */
/* arg value (TFP_TV *) */

#define RDEVENTTV  45          /* Read Event, Unlatched, Binary */
/* arg value (TFP_TV *) */

#define RDTIMETMREQ  46          /* Read Time, Latched, Decimal */
/* arg value (TFP_TM *) */
 
#define RDEVENTTMREQ  47         /* Read Event, Latched Decimal */
/* arg value (TFP_TM *) */
 
#define RDTIMETVREQ  48        /* Read Time, Latched, Binary */
/* arg value (TFP_TV *) */ 

#define RDEVENTTVREQ 49        /* Read Event, Latched, Binary */
/* arg value (TFP_TV *) */ 

#define SETSTROBETM  50
/* arg value (TFP_TM *) */

#define SETSTROBETV  51
/* arg value (TFP_TV *) */

#define SELFREQUENCYOUT 52
/* arg values */
#define      FREQ_10MHZ  1
#define      FREQ_5MHZ  2
#define      FREQ_1MHZ  3

#define CONTROLEVENT 53
/* arg values */
#define      EVENT_RISING  1
#define      EVENT_FALLING 2
#define      CAPLOCK_ENABLE 3
#define      CAPLOCK_DISABLE 4
#define      EVCAP_ENABLE  5
#define      EVCAP_DISABLE 6
#define      EVCAP_PERIODIC 7
#define      EVCAP_EVENT  8

#define CONTROLSTROBE 54
/* arg values */
#define      STROBE_SECUS  1
#define      STROBE_USONLY 2
#define      STROBE_ENABLE 3
#define      STROBE_DISABLE 4

#define CAPUNLOCK  55
/* arg value - na */

#define RDINTSTAT  56
/* arg - (UINT32 *) */

#define SETINTSTAT  57
/* arg - (UINT32) */

#define SETINTMASK 58          /* Set Interrupt Mask Register */
/* arg - (UINT32) */
 
#define RDINTMASK 59           /* Read Interrupt Mask Register */
/* arg - (UINT32 *) */



/**************************************************************/
/* IOCTL Interrupt Service Commands                           */ 
/**************************************************************/

#define BC635_INTHOOK 70       /* Designates a routine to be called */
                               /* when an interrupt occurs.  The    */
                               /* routine is called with a single   */
                               /* parameter (TFP_DEV *).            */
/* arg - (VOIDFUNCPTR) */

#define BC635_INT_WAITONGPS 71  /* Tasks waits till GPS int  */
/* arg - na */

#define BC635_INT_WAITONEVENT 72 /* Task waits till EVENT int */
/* arg - na */

#define BC635_INT_WAITONSTROBE 73 /* Task waits till STROBE int */
/* arg - na */

#define BC635_INT_WAITON1PPS 74   /* Tasks waits till 1PPS int */
/* arg - na */

#define BC635_INT_WAITONPERIODIC 75 /* Tasks waits till PERIODIC int */
/* arg - na */

/**************************************************************/
/* IOCTL GPS Commands                                         */ 
/**************************************************************/




/*-------------------------------------------------------------*/
/* bc635PCI STATUS bits (from TIME0, EVENT0 registers) */

#define STAT_FLYWHEELING 0x01
#define STAT_TIME_OFFSET 0x02
#define STAT_FREQ_OFFSET 0x04

/*-------------------------------------------------------------*/
/* bc635PCI CONRTOL register bits */

#define CTRL_LOCKEN   0x01
#define CTRL_EVSOURCE  0x02
#define CTRL_EVSENSE  0x04
#define CTRL_EVENTEN  0x08
#define CTRL_STREN   0x10
#define CTRL_STRMODE  0x20
#define CTRL_F10   0x00
#define CTRL_F5    0x40
#define CTRL_F1    0x80
#define CTRL_FMASK   0xC0

/*-------------------------------------------------------------*/
/* bc635PCI ACK register bits */

#define ACK_COMMAND  0x01
#define ACK_PACKET  0x04
#define ACK_ACTION  0x80

/*-------------------------------------------------------------*/
/* bc635PCI INTSTAT and MASK register bits */

#define BCINT_EVENT  0x01
#define INT_PERIODIC 0x02
#define INT_STROBE  0x04
#define INT_1PPS  0x08
#define INT_PACKET  0x10
#define INT_BITMASK  0x1F


#define SW_RESET  1

#ifdef __cplusplus
}

#endif

#endif                          /* __INCbc635h */
