/* bc635P.h - Datum PCI Time/Frequency Processor Card header defs (private) */

/*
  *******************************************************************************
** Copyright (C) 2000 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/


/*
modification history
--------------------
12jun00,spk: changed macro names with 'BCD' in them to 'DEC' for decimal. 
*/

#ifndef __INCbc635Ph
#define __INCbc635Ph

#ifdef __cplusplus
extern  "C"
{
#endif

#include "iosLib.h"
#include "errnoLib.h"
#include "semLib.h"
#include "bc635.h"

/* macros */

#define TFP_MSG           \
    logMsg

#define TFP_DEBUG_MSG     \
    if (bc635Debug)         \
    TFP_MSG

#define TFP_INT_DEBUG_MSG \
    if (bc635IntsDebug)     \
    TFP_MSG


/* vxWorks device header for bc635/7(TFP) pci device */

    typedef struct
    {
        DEV_HDR devHdr;         /* I/O device header */
       
        int     gps_supported;  /* TRUE for BC637 */

        VOIDFUNCPTR inthook;    /* User Interrupt hook routine */   

        SEM_ID  datamutex;      /* to prevent corruption of this data structure */
        SEM_ID  eventmutex;     /* to maintain a consistent latch */
        SEM_ID  timemutex;      /* to maintain a consistent latch */
        SEM_ID  cmdmutex;       /* to prevent command corruption */
        SEM_ID  eventSem;       /* one sem for each int source */
        SEM_ID  periodicSem;
        SEM_ID  strobeSem;
        SEM_ID  onePpsSem;
        SEM_ID  gpsSem;

        UINT32  expectedInts;   /* Interrupts expected by driver */

        UINT32  baseAdrsRegs;   /* base addr of card regs as seen from cpu */
        UINT32  baseAdrsDpram;  /* base addr of card dpram as seen from cpu */

        UINT32  baseAdrsInput;  /* base addr of dpram 'input' area */
        UINT32  baseAdrsOutput; /* base addr of dpram 'output' area */
        UINT32  baseAdrsGpsPkt; /* base addr of dpram 'gps packet' area */
        UINT32  baseAdrsYr;     /* base addr of dpram 'year area' */

    }       TFP_DEV;


/*
 * Device ID - Datum BC635/7 device id
 * Vendor ID - Datum BC635/7 PCI SIG allocated vendor id
 */

#define BC635_PCI_DEVICE_ID  0x4013
#define BC635_PCI_VENDOR_ID  0x12E2


#ifdef  _ASMLANGUAGE
#define CASTINT
#else
#define CASTINT (unsigned int *)
#endif


#define TFP_DPRAM_SIZE 0x1000   /* size in bytes of dpram space */

/*
 * Offset in bytes from BAR0 to start of dpram (BAR1).
 * Therefore, if BAR0(register base) starts at addr X, then
 * BAR1(dpram base) starts at (BAR0 + TFP_DPRAM_ADRS_START_OFFSET).
 */
#define TFP_DPRAM_ADRS_START_OFFSET  0x1000

/*
 * Note on register and dpram accesses.
 *   - Register space registers are read/written as 4-bytes.
 *     So accesses must be done by the TFP_OUT_LONG and
 *     TFP_IN_LONG macros.
 *
 *   - DPRAM requires byte access always.
 *     So accesses must be done by the TFP_OUT_BYTE and
 *     TFP_IN_BYTE macros.
 */

#ifndef TFP_ADRS                /* to permit alternative board addressing */
#define TFP_ADRS(base,reg)   (CASTINT (base + reg))
#endif                          /* TFP_ADRS */

#define TFP_OUT_LONG(adr,val) \
    sysPciWrite32((UINT32)(adr),(UINT32)(val))

#define TFP_IN_LONG(adr,pVal) \
    sysPciRead32((UINT32)(adr),(UINT32 *)(pVal))

#define TFP_OUT_BYTE(adr,val) \
    sysPciOutByte((UINT32)(adr),(UINT8)(val))

#define TFP_IN_BYTE(adr) \
    sysPciInByte((UINT32)(adr))



/* macros to determine TFP register addresses */

#define TFP_REG_TIMEREQ(base)       TFP_ADRS(base, 0x00)
#define TFP_REG_EVENTREQ(base)      TFP_ADRS(base, 0x04)
#define TFP_REG_UNLOCK(base)        TFP_ADRS(base, 0x08)
#define TFP_REG_CONTROL(base)       TFP_ADRS(base, 0x10)
#define TFP_REG_ACK(base)           TFP_ADRS(base, 0x14)
#define TFP_REG_INTMASK(base)       TFP_ADRS(base, 0x18)
#define TFP_REG_INTSTAT(base)       TFP_ADRS(base, 0x1c)
#define TFP_REG_MINSTRB(base)       TFP_ADRS(base, 0x20)
#define TFP_REG_MAJSTRB(base)       TFP_ADRS(base, 0x24)
#define TFP_REG_TIME0(base)         TFP_ADRS(base, 0x30)
#define TFP_REG_TIME1(base)         TFP_ADRS(base, 0x34)
#define TFP_REG_EVENT0(base)        TFP_ADRS(base, 0x38)
#define TFP_REG_EVENT1(base)        TFP_ADRS(base, 0x3c)


/* TFP macros to determine dpram addresses */

#define TFP_DPR_IN_ADRS_LSB(base)  TFP_ADRS(base,(TFP_DPRAM_SIZE-1))
#define TFP_DPR_IN_ADRS_MSB(base)  TFP_ADRS(base,(TFP_DPRAM_SIZE-2))
#define TFP_DPR_OUT_ADRS_LSB(base) TFP_ADRS(base,(TFP_DPRAM_SIZE-3))
#define TFP_DPR_OUT_ADRS_MSB(base) TFP_ADRS(base,(TFP_DPRAM_SIZE-4))
#define TFP_DPR_GPS_ADRS_LSB(base) TFP_ADRS(base,(TFP_DPRAM_SIZE-5))
#define TFP_DPR_GPS_ADRS_MSB(base) TFP_ADRS(base,(TFP_DPRAM_SIZE-6))
#define TFP_DPR_YR_ADRS_LSB(base)  TFP_ADRS(base,(TFP_DPRAM_SIZE-7))
#define TFP_DPR_YR_ADRS_MSB(base)  TFP_ADRS(base,(TFP_DPRAM_SIZE-8))


/* Request TFP Data Sub-commands */

#define TFP_MODEL_LEN              9    /* num 'model' chars returned by tfp */


/* TFP 'ack' register protocol values */

#define TFP_ACK_CLR       0x01  /* write this to 'ack' reg to clear */
#define TFP_ACK_GPS_CLR   0x04
#define TFP_ACK_CMDRDY    0x80  /* indicates command is ready to tfp */
#define TFP_ACK_COMPLETE  0x01  /* indicates that tfp has completed command */
#define TFP_ACK_GPS_COMPLETE 0x04 /* indicates gps complete */


/* Macros for minor reg (time0) binary values */

#define TFP_TM0_BUSEC(m)   (m & 0x000fffff)     /* b0-b19 */
#define TFP_TM0_HNSEC(m)  ((m & 0x00f00000) >> 20)      /* b20-b23 */
#define TFP_TM0_STATUS(m) ((m & 0x0f000000) >> 24)      /* b24-b27 */

/* Macros for decimal major,minor time values */

#define TFP_TM1_DEC_SEC(m)   (m & 0x0000003f)   /* 6 bits,TM1:b0-b5 */
#define TFP_TM1_DEC_MIN(m)  ((m & 0x00003f00) >> 8)     /* 5 bits,TM1:b8-b12 */
#define TFP_TM1_DEC_HOUR(m) ((m & 0x001f0000) >> 16)    /* 5 bits,TM1:b16-b20 */

/* 9 bits, bits 0-7 => TM1:b24-31, bit 8 => TM0:b28 */
#define TFP_TM_DEC_YDAY(m0,m1)  (((m1 & 0xff000000) >> 24) + \
                                 ((m0 & 0x10000000) >> 20))


/* bc635PCI DPRAM Commands */
 
#define COM_SETTIMINGMODE 0x10
#define COM_SETTIMEFORMAT 0x11
#define COM_SETTIME   0x12
#define COM_SETYEAR   0x13
#define COM_SETPERIODIC  0x14
#define COM_SETTCFORMAT  0x15
#define COM_SETTCMOD  0x16
#define COM_SETTIMINGOFFSET 0x17
#define COM_REQUTCTMDATA 0x18   /* sd */
#define COM_GETDATA   0x19
#define GETDATA_DAC  0
#define GETDATA_VERSION 2
#define COM_SOFTRESET  0x1A     /* sd */
#define COM_SETTCOUTFMT  0x1B
#define COM_SETGENTMOFF  0x1C
#define COM_SETLOCTMOFF  0x1D
#define COM_SETLEAPSEC  0x1E
#define COM_SELCLOCK  0x20
#define SELCLOCK_INT  'I'
#define SELCLOCK_EXT  'E'
#define COM_JAMSYNCENABLE 0x21
#define JAMSYNC_DIS  0
#define JAMSYNC_EN  1
#define COM_FORCEJAMSYNC 0x22
#define COM_DISCENABLE  0x23
#define DISC_DIS   0
#define DISC_EN   1
#define COM_SETDAC   0x24
#define COM_SETDISCGAIN  0x25
#define COM_REQBATTSTAT  0x26
#define COM_SYNCRTC   0x27
#define COM_DISCBATTRTC  0x28
#define COM_SETCLKVAL  0x29
#define COM_SENDGPSPACKET 0x30
#define COM_REQGPSPACKET 0x31
#define COM_MANREQGPSPACKET 0x32
#define COM_GPSTIMEFORMAT 0x33
#define COM_SETGPSMDFLG  0x34
#define COM_SETLOCTMFLG  0x40
#define COM_REQDAYLTFLG  0x41
#define COM_SETYRINCFLG  0x42
#define COM_REQDTFWNUM  0x4F
#define COM_REQASSEMB  0xF4
#define COM_REQHWFAB  0xF5
#define COM_REQTFPMODEL  0xF6
#define COM_REQSERNUM  0xFE




/* declarations */

    IMPORT void sysPciWrite32 (UINT32 address, UINT32 val);
    IMPORT UINT8 sysPciInByte (UINT32 address);
    IMPORT void sysPciOutByte (UINT32 address, UINT8 val);
    IMPORT UINT16 sysPciInWord (UINT32 address);
    IMPORT UINT32 sysPciInLong (UINT32 address);
    IMPORT void sysPciRead32 (UINT32 address, UINT32 * pVal);

    IMPORT void tfpDrvFree (TFP_DEV * pDev);
    IMPORT TFP_DEV *tfpOpen (TFP_DEV * pDev, char *remainder, int mode);
    IMPORT STATUS tfpClose (TFP_DEV * pDev);
    IMPORT STATUS tfpIoctl (TFP_DEV * pDev, int ioReqCode, int arg);


    IMPORT STATUS tfpCmdSend (TFP_DEV * pDev);
    IMPORT int tfpGetOutBytes (UINT32 outputAdrs, u_char * pDest, int len);
    IMPORT STATUS tfpCmdReqDataSend (TFP_DEV * pDev, UINT8 reqDataCode);

    IMPORT STATUS tfpCmdGetTfpModel (TFP_DEV * pDev, char *str);
    IMPORT STATUS tfpCmdSetTmMode (TFP_DEV * pDev, int mode);
    IMPORT STATUS tfpCmdSetTmFmt (TFP_DEV * pDev, int fmt);
    IMPORT STATUS tfpCmdSetTcFmt (TFP_DEV * pDev, int fmt);
    IMPORT STATUS tfpCmdSetTcm (TFP_DEV * pDev, int type);
    IMPORT STATUS tfpCmdSetDelay (TFP_DEV * pDev, int offset);
    IMPORT STATUS tfpCmdSetDecTime (TFP_DEV * pDev, TFP_TM * pTm);
    IMPORT STATUS tfpCmdSetBinTime (TFP_DEV * pDev, TFP_TV * pTv);
    IMPORT STATUS tfpCmdSetYr (TFP_DEV * pDev, int year);
    IMPORT STATUS tfpCmdSoftwareReset (TFP_DEV * pDev);


#ifdef __cplusplus
}

#endif

#endif                          /* __INCbc635Ph */
